@@ -8,6 +8,7 @@
  *
  *  policy daemon is used in conjuction with postfix to combat spam.
  *
+ *  Copyright (C) 2007 Nigel Kukard <nkukard@lbsd.net>
  *  Copyright (C) 2004 Cami Sardinha (cami@mweb.co.za)
  *
  *
@@ -152,6 +153,8 @@ main(int argc, char **argv)
 
     if(FD_ISSET(listenfd, &rset))                     /* new client connection */
     {
+      int found_free_slot = 0;
+
       clilen=sizeof(cliaddr);
       connfd=w_accept(listenfd, (struct sockaddr *) &cliaddr, &clilen);
 
@@ -160,10 +163,21 @@ main(int argc, char **argv)
           client[numi] = connfd;                       /* save file descriptor */
           if(DEBUG > 0)
             logmessage("DEBUG: saved fd: numi = %d, connfd = %d\n", numi, connfd);
+          found_free_slot = 1;
           break;
         }
       }
 
+      /* check if we ran out of slots and didn't find one above */
+      if (!found_free_slot)
+      {
+        logmessage("WARNING: No free slots found, closing connection from %s:%d\n",
+          w_inet_ntop(AF_INET, &cliaddr.sin_addr, host, sizeof(host)),
+          ntohs(cliaddr.sin_port));
+        w_close(connfd);
+        continue;
+      }
+
       /* tcp acl check */
 /*
       if(w_tcp_conn_acl(w_inet_ntop(AF_INET, &cliaddr.sin_addr, host, sizeof(host))) == -1)
@@ -176,16 +190,6 @@ main(int argc, char **argv)
 */
       /* max fds */
 
-      if(numi==MAXFDS)
-      {
-        logmessage("connection from: %s port: %d slots: %d of %d used (**WARNING**)\n",
-          w_inet_ntop(AF_INET, &cliaddr.sin_addr, host, sizeof(host)),
-          ntohs(cliaddr.sin_port), numi, MAXFDS);
-
-        w_close(connfd);
-        continue;
-      }
-
       logmessage("connection from: %s port: %d slots: %d of %d used\n",
         w_inet_ntop(AF_INET, &cliaddr.sin_addr, host, sizeof(host)),
         ntohs(cliaddr.sin_port), numi, MAXFDS);
@@ -216,10 +220,11 @@ main(int argc, char **argv)
           logmessage("DEBUG: fd: %d select(): fd %d is ready for read\n", sockfd, sockfd);
      
         /* read as much data as we can */
-	rres = w_read(sockfd,buf[sockfd]);
+        rres = w_read(sockfd,buf[sockfd],MAXLINE);
 	switch (rres)
 	{
-	  case -1:
+          case -3:
+          case -1:
             w_close(sockfd);            /* shut down socket           */
             FD_CLR(sockfd, &rallset);   /* remove fd from read set    */
             client[numi] = -1;          /* make descriptor available  */
